local L = LibStub('AceLocale-3.0'):GetLocale('TimeToDie', true)

local TimeToDie = LibStub('AceAddon-3.0'):NewAddon('TimeToDie', 'AceEvent-3.0', 'AceConsole-3.0')
_G.TimeToDie = TimeToDie

------------------------------
--      Initialization      --
------------------------------

local defaultFont = GameFontNormal:GetFont()
local events = {target = 'PLAYER_TARGET_CHANGED', focus = 'PLAYER_FOCUS_CHANGED'}
local healing = "|cff00ff00"..SHOW_COMBAT_HEALING .."|r"

local defaults = {
   profile = {
      locked = false,
      enemy = true,
      target = 'target',
      font = defaultFont,
      size = 24,
      r = 1,
      g = .82,
      b = 0,
      outline = '',
      strata = 'LOW',
      update = 1,
   }
}

function TimeToDie:OnInitialize()
   local db = LibStub('AceDB-3.0'):New('TimeToDieDB', defaults, 'Default')
   self.db = db
   local RegisterCallback = db.RegisterCallback
   RegisterCallback(self, 'OnProfileChanged', 'ApplySettings')
   RegisterCallback(self, 'OnProfileCopied', 'ApplySettings')
   RegisterCallback(self, 'OnProfileReset', 'ApplySettings')

   self:RegisterOptions()
end

function TimeToDie:OnEnable()
	self:RegisterEvent('PLAYER_TARGET_CHANGED', 'OnChange')
	self:RegisterEvent('PLAYER_FOCUS_CHANGED', 'OnChange')
	self:RegisterEvent('UNIT_HEALTH')

	self:ApplySettings()

	TimeToDieFrame:Show()
end

function TimeToDie:OnDisable()
   TimeToDieFrame:Hide()
end

-------------------------------
--      Event Functions      --
-------------------------------

local health0, time0 -- initial health and time point
local mhealth, mtime -- current midpoint

function TimeToDie:UNIT_HEALTH(event, unit)
   local p = self.db.profile
	if unit ~= p.target then return end
   if p.enemy and (UnitIsFriend('player', p.target) or UnitIsPlayer(p.target)) then return end

	local health, time = UnitHealth(p.target), GetTime()

	if not health0 then
		health0, time0 = health, time
		return
	end
	if mhealth then
		mhealth, mtime = (mhealth + health) / 2, (mtime + time) / 2
	else
		mhealth, mtime = health, time
	end

	local dhealth = mhealth - health0

   if health == UnitHealthMax(p.target) then
      TimeToDieFrame.text:SetText(not p.locked and "TimeToDie" or nil)
      health0, time0, mhealth, mtime = nil
	elseif dhealth ~= 0 then
		time = health * (time0 - mtime) / dhealth
      if time < 0 then
         TimeToDieFrame.text:SetText(healing)
         health0, time0, mhealth, mtime = nil
		elseif time > 60 then
			TimeToDieFrame.text:SetText(format("%dm %ds", time/60, time%60))
		else
			TimeToDieFrame.text:SetText(format("%ds", time%60))
		end
	else
		TimeToDieFrame.text:SetText("0s")
	end
end

function TimeToDie:OnChange(event)
   local p = self.db.profile
	if events[p.target] ~= event then return end
	TimeToDieFrame.text:SetText(not p.locked and "TimeToDie" or nil)
	health0, time0, mhealth, mtime = nil
end

------------------------------
--      Core Functions      --
------------------------------

--~ local p = self.db.profile
--~ frame = TimeToeDieFrame or CreateFrame('frame', 'TimeToDieFrame', UIParent)
--~ frame:RegisterForDrag('LeftButton')
--~ frame:SetClampedToScreen(true)
--~ frame:SetMovable(true)
--~ frame:SetWidth(120)
--~ frame:SetHeight(30)

--~ frame.isMoving = false
--~ frame:SetScript('OnDragStart', function(frame)
--~    frame:StartMoving()
--~    frame.isMoving = true
--~ end)
--~ frame:SetScript('OnDragStop', function(frame)
--~    if frame.isMoving then
--~       frame:StopMovingOrSizing()
--~       frame.isMoving = false
--~       local p = TimeToDie.db.profile
--~       p.p1, _, p.p2, p.x, p.y = frame:GetPoint()
--~    end
--~ end)

--~ bg = frame.texture or frame:CreateTexture("TimeToDieFrameTexture", 'BACKGROUND')
--~ bg:SetTexture(0, 0, 0, 0.5)
--~ bg:ClearAllPoints()
--~ bg:SetAllPoints(frame)
--~ frame.bg = bg

--~ text = frame.text or frame:CreateFontString("TimeToDieFrameText", 'OVERLAY', 'GameFontNormal')
--~ text:SetFont(p.font, p.size, p.outline)
--~ text:SetJustifyH('CENTER')
--~ text:SetPoint('CENTER', frame)
--~ frame.text = text

--~ 	if dewdrop then
--~ 		dewdrop:Register(frame, 'children', function()
--~ 			dewdrop:FeedAceOptionsTable(options)
--~ 		end)
--~ 	end

--~ frame:Hide()

TimeToDie.dataobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("TimeToDie", {
   type = "data source",
   text = "No Target",
   value = 0,
   suffix = "s",
   label = 'TimeToDie',
   unit = "time",
   icon = 'Interface\\Icons\\Spell_Holy_BorrowedTime',
})
